@@ -43,6 +43,7 @@ struct chirp_file {
 
 struct hash_table *table = 0;
 static int chirp_reli_blocksize = 65536;
+static int chirp_reli_default_nreps = 0;
 
 INT64_T chirp_reli_blocksize_get()
 {
@@ -73,6 +74,13 @@ static struct chirp_client * connect_to_host( const char *host, time_t stoptime
 	}
 
 	if(c) {
+		/*
+		If a default replication factor was set earlier, then
+		it must be re-applied when re-connecting after a failure.
+		*/
+		if(chirp_reli_default_nreps>0) {
+			chirp_client_setrep(c,"@@@",chirp_reli_default_nreps,stoptime);
+		}
 		hash_table_insert(table,host,c);
 		return c;
 	} else {
@@ -674,6 +682,17 @@ INT64_T chirp_reli_md5( const char *host, const char *path, unsigned char digest
 	RETRY_ATOMIC( result = chirp_client_md5(client,path,digest,stoptime); )
 }
 
+INT64_T chirp_reli_setrep(const char *host, const char *path, int nreps, time_t stoptime )
+{
+	/*
+	If setting a default replication factor for the session,
+	then save it in a global so that it can be re-applied on a re-connection.
+	*/
+
+	if(!strcmp(path,"@@@")) chirp_reli_default_nreps = nreps;
+	RETRY_ATOMIC( result = chirp_client_setrep(client,path,nreps,stoptime); )
+}
+
 INT64_T chirp_reli_remote_debug( const char *host, const char *flag, time_t stoptime )
 {
 	RETRY_ATOMIC( result = chirp_client_remote_debug(client,flag,stoptime); )
@@ -694,46 +713,6 @@ INT64_T chirp_reli_thirdput( const char *host, const char *path, const char *thi
 	RETRY_ATOMIC( result = chirp_client_thirdput( client, path, thirdhost, thirdpath, stoptime ); )
 }
 
-INT64_T chirp_reli_group_create( const char *host, char *group, time_t stoptime )
-{
-	RETRY_ATOMIC ( result = chirp_client_group_create(client,group,stoptime); )
-}
-
-INT64_T chirp_reli_group_list( const char *host, const char *group, chirp_dir_t callback, void *arg, time_t stoptime )
-{
-	RETRY_ATOMIC ( result = chirp_client_group_list(client,group,callback,arg,stoptime); )
-}
-
-INT64_T chirp_reli_group_add( const char *host, char *group, char *user, time_t stoptime )
-{
-	RETRY_ATOMIC ( result = chirp_client_group_add(client,group,user,stoptime); )
-}
-
-INT64_T chirp_reli_group_remove( const char *host, char *group, char *user, time_t stoptime )
-{
-	RETRY_ATOMIC ( result = chirp_client_group_remove(client,group,user,stoptime); )
-}
-
-INT64_T chirp_reli_group_lookup(const char* host, const char* group, const char* user, time_t stoptime)
-{
-        RETRY_ATOMIC( result = chirp_client_group_lookup( client, group, user, stoptime ); )
-}
-
-INT64_T chirp_reli_group_policy_set( const char *host, char *group, unsigned long int file_duration, unsigned long int dec_duration, time_t stoptime )
-{
-	RETRY_ATOMIC ( result = chirp_client_group_policy_set(client,group,file_duration,dec_duration,stoptime); )
-}
-
-INT64_T chirp_reli_group_cache_update( const char* host, const char* group, time_t mod_time, time_t stoptime )
-{
-        RETRY_ATOMIC( result = chirp_client_group_cache_update( client, group, mod_time, stoptime ); )
-} 
-
-INT64_T chirp_reli_group_policy_get(const char* host, const char* group, int* policy, int* file_duration, int* dec_duration, time_t stoptime)
-{
-        RETRY_ATOMIC( result = chirp_client_group_policy_get( client, group, policy, file_duration, dec_duration, stoptime ); )
-}
-
 INT64_T chirp_reli_mkalloc( const char *host, const char *path, INT64_T size, INT64_T mode, time_t stoptime )
 {
 	RETRY_ATOMIC( result = chirp_client_mkalloc(client,path,size,mode,stoptime); )
@@ -744,36 +723,6 @@ INT64_T chirp_reli_lsalloc( const char *host, const char *path, char *allocpath,
 	RETRY_ATOMIC( result = chirp_client_lsalloc(client,path,allocpath,total,inuse,stoptime); )
 }
 
-INT64_T chirp_reli_job_begin( const char *host, const char *cwd, const char *input, const char *output, const char *error, const char *cmdline, time_t stoptime )
-{
-	RETRY_ATOMIC( result = chirp_client_job_begin(client,cwd,input,output,error,cmdline,stoptime); )
-}
-
-INT64_T chirp_reli_job_commit( const char *host, INT64_T jobid, time_t stoptime )
-{
-	RETRY_ATOMIC( result = chirp_client_job_commit(client,jobid,stoptime); );
-}
-
-INT64_T chirp_reli_job_wait( const char *host, INT64_T jobid, struct chirp_job_state *state, int wait_time, time_t stoptime )
-{
-	RETRY_ATOMIC( result = chirp_client_job_wait(client,jobid,state,wait_time,stoptime); );
-}
-
-INT64_T chirp_reli_job_kill( const char *host, INT64_T jobid, time_t stoptime )
-{
-	RETRY_ATOMIC( result = chirp_client_job_kill(client,jobid,stoptime); );
-}
-
-INT64_T chirp_reli_job_remove( const char *host, INT64_T jobid, time_t stoptime )
-{
-	RETRY_ATOMIC( result = chirp_client_job_remove(client,jobid,stoptime); );
-}
-
-INT64_T chirp_reli_job_list( const char *host, chirp_joblist_t callback, void *arg, time_t stoptime )
-{
-	RETRY_ATOMIC( result = chirp_client_job_list(client,callback,arg,stoptime); );
-}
-
 struct chirp_dir {
        struct chirp_dirent *head;
        struct chirp_dirent *current;
