@@ -86,24 +86,13 @@ static INT64_T do_help(int argc, char **argv);
 static INT64_T do_quit(int argc, char **argv);
 static INT64_T do_whoami(int argc, char **argv);
 static INT64_T do_whoareyou(int argc, char **argv);
-static INT64_T do_job_run(int argc, char **argv);
-static INT64_T do_job_start(int argc, char **argv);
-static INT64_T do_job_exec(int argc, char **argv);
-static INT64_T do_job_wait(int argc, char **argv);
-static INT64_T do_job_list(int argc, char **argv);
-static INT64_T do_job_kill(int argc, char **argv);
-static INT64_T do_job_remove(int argc, char **argv);
 static INT64_T do_md5(int argc, char **argv);
+static INT64_T do_setrep(int argc, char **argv);
 static INT64_T do_localpath(int argc, char **argv);
 static INT64_T do_audit(int argc, char **argv);
 static INT64_T do_thirdput(int argc, char **argv);
 static INT64_T do_mkalloc(int argc, char **argv);
 static INT64_T do_lsalloc(int argc, char **argv);
-static INT64_T do_group_create(int argc, char **argv);
-static INT64_T do_group_add(int argc, char **argv);
-static INT64_T do_group_remove(int argc, char **argv);
-static INT64_T do_group_list(int argc, char **argv);
-static INT64_T do_group_policy(int argc, char **argv);
 static INT64_T do_matrix_create(int argc, char **argv);
 static INT64_T do_matrix_list(int argc, char **argv);
 static INT64_T do_matrix_delete(int argc, char **argv);
@@ -162,22 +151,11 @@ static struct command list[] = {
 	{"df", 1, 0, 1, "[-k|-m|-g|-t]", do_statfs},
 	{"chmod", 1, 2, 2, "<mode> <path>", do_chmod},
 	{"md5", 1, 1, 1, "<path>", do_md5},
+	{"setrep", 1, 2, 2, "<path> <nreps>", do_setrep},
 	{"localpath", 1, 0, 1, "[remotepath]", do_localpath},
 	{"audit", 1, 0, 1, "[-r]", do_audit},
 	{"lsalloc", 1, 0, 1, "[path]", do_lsalloc},
 	{"mkalloc", 1, 2, 2, "<path> <size>", do_mkalloc},
-	{"job_run", 1, 1, 100, "<command> [args] [<stdin] [>stdout] [>&stderr]", do_job_run},
-	{"job_exec", 1, 4, 100, "<infile> <outfile> <errfile> <command> [args]", do_job_exec},
-	{"job_start", 1, 4, 100, "<infile> <outfile> <errfile> <command> [args]", do_job_start},
-	{"job_list", 1, 0, 0, "", do_job_list},
-	{"job_wait", 1, 1, 1, "<jobid>", do_job_wait},
-	{"job_kill", 1, 1, 1, "<jobid>", do_job_kill},
-	{"job_remove", 1, 1, 1, "<jobid>", do_job_remove},
-	{"group_create", 1, 1, 1, "<groupname>", do_group_create},
-	{"group_add", 1, 2, 2, "<groupname> <username>", do_group_add},
-	{"group_remove", 1, 2, 2, "<groupname> <username>", do_group_remove},
-	{"group_list", 1, 1, 1, "<groupname>", do_group_list},
-	{"group_policy", 1, 1, 5, "-f <file cache duration> -d <decision cache duration> <groupname>", do_group_policy},
 	{"matrix_create", 1, 4, 4, "<path> <width> <height> <nhosts>", do_matrix_create},
 	{"matrix_list", 1, 1, 1, "<path>", do_matrix_list},
 	{"matrix_delete", 1, 1, 1, "<path>", do_matrix_delete},
@@ -985,211 +963,6 @@ static INT64_T do_whoareyou(int argc, char **argv)
 	return result;
 }
 
-static INT64_T last_job_started = 0;
-
-static INT64_T do_job_start(int argc, char **argv)
-{
-	char infile[CHIRP_LINE_MAX];
-	char outfile[CHIRP_LINE_MAX];
-	char errfile[CHIRP_LINE_MAX];
-	char cmd[CHIRP_LINE_MAX];
-	int i;
-	INT64_T jobid;
-
-	if(!strcmp(argv[1], "-")) {
-		strcpy(infile, argv[1]);
-	} else {
-		complete_remote_path(argv[1], infile);
-	}
-
-	if(!strcmp(argv[2], "-")) {
-		strcpy(outfile, argv[2]);
-	} else {
-		complete_remote_path(argv[2], outfile);
-	}
-
-	if(!strcmp(argv[3], "-")) {
-		strcpy(errfile, argv[3]);
-	} else {
-		complete_remote_path(argv[3], errfile);
-	}
-
-	if(argv[4][0] == '@') {
-		strcpy(cmd, argv[4]);
-	} else {
-		complete_remote_path(argv[4], cmd);
-	}
-
-	for(i = 5; i < argc; i++) {
-		strcat(cmd, " ");
-		strcat(cmd, argv[i]);
-	}
-
-	int result = 1;
-
-	jobid = chirp_reli_job_begin(current_host, current_remote_dir, infile, outfile, errfile, cmd, stoptime);
-	if(jobid > 0) {
-		printf("jobid %lld created\n", jobid);
-		if(chirp_reli_job_commit(current_host, jobid, stoptime) == 0) {
-			printf("jobid %lld submitted.\n", jobid);
-			last_job_started = jobid;
-			result = 0;
-		} else {
-			printf("jobid %lld could not be started: %s\n", jobid, strerror(errno));
-		}
-	} else {
-		printf("job could not be created: %s\n", strerror(errno));
-	}
-
-	return result;
-}
-
-static INT64_T do_job_wait_jobid(INT64_T jobid)
-{
-	struct chirp_job_state status;
-
-	while(1) {
-		if(chirp_reli_job_wait(current_host, jobid, &status, 300, stoptime) == 0) {
-			switch (status.state) {
-			case CHIRP_JOB_STATE_BEGIN:
-			case CHIRP_JOB_STATE_IDLE:
-			case CHIRP_JOB_STATE_RUNNING:
-			case CHIRP_JOB_STATE_SUSPENDED:
-				continue;
-
-			case CHIRP_JOB_STATE_COMPLETE:
-				printf("jobid %lld completed with exit code %d\n", jobid, status.exit_code);
-				break;
-
-			case CHIRP_JOB_STATE_KILLED:
-			case CHIRP_JOB_STATE_FAILED:
-			default:
-				printf("jobid %lld %s\n", jobid, chirp_job_state_string(status.state));
-				break;
-			}
-			chirp_reli_job_remove(current_host, jobid, stoptime);
-			printf("jobid %lld removed.\n", jobid);
-			break;
-		} else {
-			printf("jobid %lld error: %s\n", jobid, strerror(errno));
-			break;
-		}
-	}
-	return 0;
-}
-
-static INT64_T do_job_wait(int argc, char **argv)
-{
-	return do_job_wait_jobid(atoll(argv[1]));
-}
-
-static INT64_T do_job_exec(int argc, char **argv)
-{
-	if(do_job_start(argc, argv) == 0) {
-		do_job_wait_jobid(last_job_started);
-	}
-	return 0;
-}
-
-static INT64_T do_job_kill(int argc, char **argv)
-{
-	return chirp_reli_job_kill(current_host, atoll(argv[1]), stoptime);
-}
-
-static INT64_T do_job_remove(int argc, char **argv)
-{
-	return chirp_reli_job_remove(current_host, atoll(argv[1]), stoptime);
-}
-
-static void job_list_callback(struct chirp_job_state *state, void *arg)
-{
-	char submit_date[16];
-	char submit_time[16];
-	int run_time = 0;
-
-	strftime(submit_date, sizeof(submit_date), "%m/%d/%Y", localtime(&state->submit_time));
-	strftime(submit_time, sizeof(submit_time), "%T", localtime(&state->submit_time));
-
-	if(state->start_time > 0) {
-		if(state->stop_time > 0) {
-			run_time = state->stop_time - state->start_time;
-		} else {
-			run_time = time(0) - state->start_time;
-		}
-	} else {
-		run_time = 0;
-	}
-
-	if(run_time < 0)
-		run_time = 0;
-
-	int hours = run_time / 3600;
-	int minutes = (run_time % 3600) / 60;
-	int seconds = (run_time % 60);
-
-	printf("%-8lld %10s %8s %02d:%02d:%02d %-8s %3d %-16s %s\n", state->jobid, submit_date, submit_time, hours, minutes, seconds, chirp_job_state_string(state->state), state->exit_code, state->owner, state->command);
-}
-
-static INT64_T do_job_list(int argc, char **argv)
-{
-	printf("JOBID    SUB_DATE   SUB_TIME RUNTIME  STATE   EXIT OWNER            COMMAND\n");
-	return chirp_reli_job_list(current_host, job_list_callback, stdout, stoptime);
-}
-
-static INT64_T do_job_run(int argc, char **argv)
-{
-	char infile[CHIRP_LINE_MAX];
-	char outfile[CHIRP_LINE_MAX];
-	char errfile[CHIRP_LINE_MAX];
-	char cmd[CHIRP_LINE_MAX];
-	char args[CHIRP_LINE_MAX];
-	char *newargv[6];
-	int i;
-
-	strcpy(infile, "-");
-	strcpy(outfile, "stdout.txt");
-	strcpy(errfile, "stderr.txt");
-	strcpy(cmd, argv[1]);
-	args[0] = 0;
-
-	for(i = 2; i < argc; i++) {
-		if(argv[i][0] == '<') {
-			strcpy(infile, &argv[i][1]);
-		} else if(argv[i][0] == '>') {
-			if(argv[i][1] == '&') {
-				strcpy(errfile, &argv[i][2]);
-			} else {
-				strcpy(outfile, &argv[i][1]);
-			}
-		} else {
-			strcat(args, argv[i]);
-			strcat(args, " ");
-		}
-	}
-
-	newargv[1] = infile;
-	newargv[2] = outfile;
-	newargv[3] = errfile;
-	newargv[4] = cmd;
-	newargv[5] = args;
-
-	do_job_exec(6, newargv);
-
-	if(!strcmp(outfile, "stdout.txt")) {
-		char full_path[CHIRP_PATH_MAX];
-		complete_remote_path(outfile, full_path);
-		chirp_reli_getfile(current_host, full_path, stdout, stoptime);
-	}
-
-	if(!strcmp(errfile, "stderr.txt")) {
-		char full_path[CHIRP_PATH_MAX];
-		complete_remote_path(errfile, full_path);
-		chirp_reli_getfile(current_host, full_path, stdout, stoptime);
-	}
-
-	return 0;
-}
-
 static INT64_T do_md5(int argc, char **argv)
 {
 	unsigned char digest[16];
@@ -1204,6 +977,16 @@ static INT64_T do_md5(int argc, char **argv)
 	return result;
 }
 
+static INT64_T do_setrep(int argc, char **argv)
+{
+	char full_path[CHIRP_LINE_MAX];
+
+	complete_remote_path(argv[1], full_path);
+	int nreps = atoi(argv[2]);
+
+	return chirp_reli_setrep(current_host, full_path, nreps, stoptime );
+}
+
 static INT64_T do_localpath(int argc, char **argv)
 {
 	char localpath[CHIRP_LINE_MAX];
@@ -1338,122 +1121,6 @@ static INT64_T do_lsalloc(int argc, char **argv)
 	return result;
 }
 
-static INT64_T do_group_create(int argc, char **argv)
-{
-	char full_path[CHIRP_PATH_MAX];
-	int result = 0;
-
-	if(argc < 2)
-		return -1;
-
-	complete_remote_path(argv[1], full_path);
-
-	result = chirp_reli_group_create(current_host, full_path, stoptime);
-
-	if(result > 0) {
-		printf("group:%s created\n", full_path);
-	}
-
-	return result;
-}
-
-static INT64_T do_group_add(int argc, char **argv)
-{
-	char full_path[CHIRP_PATH_MAX];
-	int result = 0;
-
-	if(argc < 3)
-		return -1;
-
-	complete_remote_path(argv[1], full_path);
-
-	result = chirp_reli_group_add(current_host, full_path, argv[2], stoptime);
-
-	if(result == 0) {
-		printf("user %s added to group %s\n", argv[2], full_path);
-	}
-
-	else if(result == 1) {
-		printf("user %s already exists in group %s\n", argv[2], full_path);
-	}
-	return result;
-}
-
-static INT64_T do_group_remove(int argc, char **argv)
-{
-	char full_path[CHIRP_PATH_MAX];
-	INT64_T result = 0;
-
-	if(argc < 3)
-		return -1;
-
-	complete_remote_path(argv[1], full_path);
-
-	result = chirp_reli_group_remove(current_host, full_path, argv[2], stoptime);
-
-	if(result == 0) {
-		printf("user %s deleted from group %s\n", argv[2], full_path);
-		return 1;
-	} else if(!errno) {
-		printf("user %s not found in group %s\n", argv[2], full_path);
-		return 0;
-	} else {
-		strerror(errno);
-	}
-	return result;
-}
-
-static void show_one_user(const char *name, void *stream)
-{
-	fprintf(stream, "%s\n", name);
-}
-
-static INT64_T do_group_list(int argc, char **argv)
-{
-	static char full_path[CHIRP_PATH_MAX];
-
-	complete_remote_path(argv[1], full_path);
-
-	return chirp_reli_group_list(current_host, full_path, show_one_user, stdout, stoptime);
-}
-
-static INT64_T do_group_policy(int argc, char **argv)
-{
-	static char full_path[CHIRP_PATH_MAX];
-	int result;
-	int file_duration = 0;
-	int dec_duration = 0;
-	char c;
-
-	optind = opterr = 0;
-	while((c = getopt(argc, argv, "+d:f:D:F")) != (char) -1) {
-		switch (c) {
-		case 'd':
-			dec_duration = atoi(optarg);
-			break;
-		case 'D':
-			dec_duration = atoi(optarg);
-			break;
-		case 'f':
-			file_duration = atoi(optarg);
-			break;
-		case 'F':
-			file_duration = atoi(optarg);
-			break;
-		}
-	}
-
-	if(argv[optind] == NULL) {	/* no group name specified */
-		errno = EINVAL;
-		return -1;
-	}
-	complete_remote_path(argv[optind], full_path);
-
-	result = chirp_reli_group_policy_set(current_host, full_path, file_duration, dec_duration, stoptime);
-
-	return result;
-}
-
 static INT64_T do_matrix_create(int argc, char **argv)
 {
 	char path[CHIRP_PATH_MAX];
