@@ -677,11 +677,13 @@ int link_fd(struct link *link)
 	return link->fd;
 }
 
-#if defined(__GLIBC__) || defined(CCTOOLS_OPSYS_DARWIN)
+#ifndef SOCKLEN_T
+#if defined(__GLIBC__) || defined(CCTOOLS_OPSYS_DARWIN) || defined(CCTOOLS_OPSYS_AIX)
 #define SOCKLEN_T socklen_t
 #else
 #define SOCKLEN_T int
 #endif
+#endif
 
 int link_address_local(struct link *link, char *addr, int *port)
 {
