@@ -378,6 +378,7 @@ static void show_help(const char *cmd)
 	printf(" -o <file>      Send debugging to this file.\n");
 	printf(" -v             Show version string\n");
 	printf(" -w <size>      Set TCP window size.\n");
+	printf(" -z <size>      Set available disk space threshold (in MB) before aborting. By default no checks on available space are performed.\n");
 	printf(" -h             Show this help screen\n");
 }
 
@@ -392,6 +393,7 @@ int main(int argc, char *argv[])
 	char addr[LINK_ADDRESS_MAX];
 	UINT64_T memory_avail, memory_total;
 	UINT64_T disk_avail, disk_total;
+	UINT64_T avail_space_threshold = 0;
 	int ncpus;
 	char c;
 	char hostname[DOMAIN_NAME_MAX];
@@ -412,7 +414,7 @@ int main(int argc, char *argv[])
 
 	debug_config(argv[0]);
 
-	while((c = getopt(argc, argv, "aC:d:ihN:o:st:w:v")) != (char) -1) {
+	while((c = getopt(argc, argv, "aC:d:ihN:o:st:w:z:v")) != (char) -1) {
 		switch (c) {
 		case 'a':
 			auto_worker = 1;
@@ -449,6 +451,9 @@ int main(int argc, char *argv[])
 			w = string_metric_parse(optarg);
 			link_window_set(w, w);
 			break;
+		case 'z':
+			avail_space_threshold = string_metric_parse(optarg) * 1024 * 1024;
+			break;
 		case 'h':
 		default:
 			show_help(argv[0]);
@@ -502,6 +507,14 @@ int main(int argc, char *argv[])
 	printf("worker: working in %s\n", tempdir);
 	mkdir(tempdir, 0700);
 	chdir(tempdir);
+	
+	if (avail_space_threshold > 0) {
+		disk_info_get(".", &disk_avail, &disk_total);
+		if (disk_avail < avail_space_threshold) {
+                       	printf("worker: aborting since available disk space (%llu) lower than threshold (%llu).\n", disk_avail, avail_space_threshold);
+	                goto abort;
+                }	
+        }	
 
 	domain_name_cache_guess(hostname);
 
@@ -509,7 +522,7 @@ int main(int argc, char *argv[])
 	time_t switch_master_time = time(0) + master_timeout;
 
 	bad_masters = hash_cache_create(127, hash_string, (hash_cache_cleanup_t) free);
-
+		
 	while(!abort_flag) {
 		char line[WORK_QUEUE_LINE_MAX];
 		int result, mode, fd;
@@ -608,6 +621,14 @@ int main(int argc, char *argv[])
 				}
 				symlink(path, filename);
 			} else if(sscanf(line, "put %s %lld %o", filename, &length, &mode) == 3) {
+				if (avail_space_threshold > 0){
+					disk_info_get(".", &disk_avail, &disk_total);
+		                        if (disk_avail < avail_space_threshold) {
+                	                	debug(D_WQ, "Available disk space (%llu) lower than threshold (%llu).\n", disk_avail, avail_space_threshold);
+	                        	        goto recover;
+                        		}	
+                		}	
+
 				mode = mode | 0600;
 				char *cur_pos, *tmp_pos;
 
@@ -770,6 +791,7 @@ int main(int argc, char *argv[])
 		}
 	}
 
+      abort:	
 	printf("worker: cleaning up %s\n", tempdir);
 	sprintf(deletecmd, "rm -rf %s", tempdir);
 	system(deletecmd);
