@@ -33,6 +33,7 @@ See the file COPYING for details.
 #include "int_sizes.h"
 #include "list.h"
 #include "timestamp.h"
+#include "xmalloc.h"
 
 #define DAG_LINE_MAX 1048576
 #define SHOW_INPUT_FILES 2
@@ -101,6 +102,7 @@ struct dag_node {
 	int failure_count;
 	dag_node_state_t state;
 	const char *command;
+	const char *original_command;
 	struct dag_file *source_files;
 	struct dag_file *target_files;
 	int source_file_names_size;
@@ -640,7 +642,7 @@ void dag_log_recover( struct dag *d, const char *filename )
 		struct dag_node *p;
 		for(n=d->nodes;n;n=n->next) {
 			/* Record node information to log */
-			fprintf(d->logfile, "# NODE\t%d\t%s\n",n->nodeid,n->command);
+			fprintf(d->logfile, "# NODE\t%d\t%s\n",n->nodeid,n->original_command);
 
 			/* Record node parents to log */
 			fprintf(d->logfile, "# PARENTS\t%d",n->nodeid);
@@ -658,12 +660,15 @@ void dag_log_recover( struct dag *d, const char *filename )
 			}
 			fputc('\n', d->logfile);
 
-			/* Record node outputsto log */
+			/* Record node outputs to log */
 			fprintf(d->logfile, "# TARGETS\t%d",n->nodeid);
 			for(f=n->target_files;f;f=f->next) {
 				fprintf(d->logfile, "\t%s",f->filename);
 			}
 			fputc('\n', d->logfile);
+			
+			/* Record translated command to log */
+			fprintf(d->logfile, "# COMMAND\t%d\t%s\n",n->nodeid,n->command);
 		}
 	}
 
@@ -1052,6 +1057,7 @@ struct dag_node * dag_node_parse( struct dag *d, FILE *file, int clean_mode )
 		c+=6;
 	}
 
+	n->original_command = xstrdup(c);
 	n->command = translate_command(d, c, n->local_job);
 
 	free(line);
@@ -1159,7 +1165,10 @@ void dag_node_submit( struct dag *d, struct dag_node *n )
 		strcat(output_files,",");
 	}
 
-	batch_queue_set_options(thequeue,getenv("BATCH_OPTIONS"));
+	const char *batch_submit_options = getenv("BATCH_OPTIONS");
+	if (batch_submit_options) {
+		batch_queue_set_options(thequeue, batch_submit_options);
+	}
 
 	time_t stoptime = time(0) + dag_submit_timeout;
 	int waittime = 1;
@@ -1455,11 +1464,11 @@ int main( int argc, char *argv[] )
 	int explicit_local_jobs_max = 0;
 	int skip_afs_check = 0;
 	int preserve_symlinks = 0;
-	const char *batch_submit_options = 0;
+	const char *batch_submit_options = getenv("BATCH_OPTIONS");
 	int auto_workers = 0;
 	char line[1024];
-    int work_queue_master_mode = WORK_QUEUE_MASTER_MODE_STANDALONE;
-    int work_queue_worker_mode = WORK_QUEUE_WORKER_MODE_SHARED;
+	int work_queue_master_mode = WORK_QUEUE_MASTER_MODE_STANDALONE;
+	int work_queue_worker_mode = WORK_QUEUE_WORKER_MODE_SHARED;
 
 	debug_config(argv[0]);
 
@@ -1485,18 +1494,18 @@ int main( int argc, char *argv[] )
 			sprintf(line,"WORK_QUEUE_PRIORITY=%d",priority);
 			putenv(strdup(line));
 			break;
-        case 'a':
-            work_queue_master_mode = WORK_QUEUE_MASTER_MODE_CATALOG;
+		case 'a':
+			work_queue_master_mode = WORK_QUEUE_MASTER_MODE_CATALOG;
+			break;
+		case 'e':
+			work_queue_worker_mode = WORK_QUEUE_WORKER_MODE_EXCLUSIVE;
 			break;
-        case 'e':
-            work_queue_worker_mode = WORK_QUEUE_WORKER_MODE_EXCLUSIVE;
-            break;
 		case 'C':
 			if(!parse_catalog_server_description(optarg)) {
 				fprintf(stderr,"makeflow: catalog server should be given as HOSTNAME:PORT'.\n");
 				exit(1);
 			}
-            work_queue_master_mode = WORK_QUEUE_MASTER_MODE_CATALOG;
+			work_queue_master_mode = WORK_QUEUE_MASTER_MODE_CATALOG;
 			break;
 		case 'I':
 			display_mode = SHOW_INPUT_FILES;
@@ -1783,6 +1792,7 @@ int main( int argc, char *argv[] )
 	}
 
 	if(batch_submit_options) {
+		debug(D_DEBUG, "setting batch options to %s\n", batch_submit_options);
 		batch_queue_set_options(remote_queue,batch_submit_options);
 	}
 
